Datasets
load_example_infection_admission_interval
load_example_infection_admission_interval() -> DataFrame
Load an example infection to hospital admission interval distribution.
This dataset contains an example infection to admission interval distribution for COVID-19.
Returns:
| Type | Description |
|---|---|
DataFrame
|
The infection to admission interval dataset |
Notes
This dataset was downloaded directly from: github.com/cdcgov/wastewater-informed-covid-forecasting
The dataset contains the following columns:
- timepoint
- probability_mass
Source code in pyrenew/datasets/infection_admission_interval.py
10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 | |
load_hospital_data_for_state
Load hospital admissions data for a specific state.
Parameters:
| Name | Type | Description | Default |
|---|---|---|---|
state_abbr
|
str
|
State abbreviation (e.g., "CA"). Default is "CA". |
'CA'
|
filename
|
str
|
CSV filename. Default is "2023-11-06.csv". |
'2023-11-06.csv'
|
Returns:
| Type | Description |
|---|---|
dict
|
Dictionary containing:
|
Notes
Data source: CDC cfa-forecast-renewal-ww repository. License: Public Domain (CC0 1.0 Universal) - U.S. Government work.
Source code in pyrenew/datasets/hospital_admissions.py
15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 | |
load_synthetic_daily_ed_visits
load_synthetic_daily_ed_visits() -> DataFrame
Load synthetic daily ED visits.
Returns:
| Type | Description |
|---|---|
DataFrame
|
Columns: date, geo_value, disease, ed_visits. |
Source code in pyrenew/datasets/synthetic_data.py
71 72 73 74 75 76 77 78 79 80 81 82 | |
load_synthetic_daily_hospital_admissions
load_synthetic_daily_hospital_admissions() -> DataFrame
Load synthetic daily hospital admissions.
Returns:
| Type | Description |
|---|---|
DataFrame
|
Columns: date, geo_value, daily_hosp_admits, pop. |
Source code in pyrenew/datasets/synthetic_data.py
57 58 59 60 61 62 63 64 65 66 67 68 | |
load_synthetic_daily_infections
load_synthetic_daily_infections() -> DataFrame
Load true daily infections and R(t) from synthetic data.
Returns:
| Type | Description |
|---|---|
DataFrame
|
Columns: date, true_infections, true_rt. |
Source code in pyrenew/datasets/synthetic_data.py
43 44 45 46 47 48 49 50 51 52 53 54 | |
load_synthetic_true_parameters
load_synthetic_true_parameters() -> dict
Load ground-truth parameters from the synthetic data generator.
Returns:
| Type | Description |
|---|---|
dict
|
True parameter values including R(t) trajectory, ascertainment rates, and delay PMFs. |
Source code in pyrenew/datasets/synthetic_data.py
29 30 31 32 33 34 35 36 37 38 39 40 | |
load_synthetic_weekly_hospital_admissions
load_synthetic_weekly_hospital_admissions() -> DataFrame
Load synthetic weekly hospital admissions.
Returns:
| Type | Description |
|---|---|
DataFrame
|
Columns: week_end, weekly_hosp_admits, location, pop. |
Source code in pyrenew/datasets/synthetic_data.py
85 86 87 88 89 90 91 92 93 94 95 96 | |
load_wastewater_data_for_state
Load wastewater data for a specific state.
Parameters:
| Name | Type | Description | Default |
|---|---|---|---|
state_abbr
|
str
|
State abbreviation (e.g., "CA"). Default is "CA". |
'CA'
|
filename
|
str
|
CSV filename. Default is "fake_nwss.csv". |
'fake_nwss.csv'
|
Returns:
| Type | Description |
|---|---|
dict
|
Dictionary containing:
|
Notes
Data source: CDC cfa-forecast-renewal-ww repository. License: Public Domain (CC0 1.0 Universal) - U.S. Government work.
The data is synthetic and contains deliberately added noise for public release. Concentrations are in copies/L and are converted to copies/mL (divided by 1000).
Source code in pyrenew/datasets/wastewater_nwss.py
15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 | |